package name.nathanmcrae.numbersstation;

import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;

public class CustomFormatter extends Formatter {
    private static final SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSXXX");

    @Override
    public String format(LogRecord record) {
        String timestamp = dateFormat.format(new Date(record.getMillis()));
        String logLevel = record.getLevel().getName();
        String className = record.getSourceClassName();
        String methodName = record.getSourceMethodName();
        String message = formatMessage(record);

        StringBuilder logMessage = new StringBuilder();
        logMessage.append(String.format("%s\t%s\t%s\t%s\t%s", timestamp, logLevel, className, methodName, message));

        if (record.getThrown() != null) {
            StringWriter sw = new StringWriter();
            PrintWriter pw = new PrintWriter(sw);
            record.getThrown().printStackTrace(pw);
            logMessage.append("\n\t").append(sw.toString());
        }

        logMessage.append(System.lineSeparator());
        return logMessage.toString();
    }
}
