package name.nathanmcrae.numbersstation;

import org.apache.commons.cli.*;

public class MainParsedArguments {
    private boolean helpFlag = false;
    private HelpFormatter formatter;
    private Options options;
    private ParseException parseException = null;
    private String stationName;
    private boolean versionFlag = false;

    public MainParsedArguments (String[] args) {
        options = new Options();

        Option help = new Option("h", "help", false, "Show help");
        options.addOption(help);

        Option version = new Option("v", "version", false, "Show version");
        options.addOption(version);

        Option station = new Option("s", "station", true, "Specify station name");
        station.setArgName("station-name");
        options.addOption(station);

        CommandLineParser parser = new DefaultParser();
        formatter = new HelpFormatter();

        try {
            CommandLine cmd = parser.parse(options, args);

            this.helpFlag = cmd.hasOption("help");
            this.versionFlag = cmd.hasOption("version");
            this.stationName = cmd.getOptionValue("station");
        } catch (ParseException e) {
            this.parseException = e;
        }
    }

    public boolean getHelpFlag() {
        return helpFlag;
    }

    public ParseException getParseException() {
        return parseException;
    }

    public String getStationName() {
        return stationName;
    }

    public boolean getVersionFlag() {
        return versionFlag;
    }

    public void printHelp() {
        formatter.printHelp("numbers-station", options);
    }
}
