package name.nathanmcrae.numbersstation;

import java.util.logging.Logger;
import java.util.logging.Level;
import java.util.Optional;
import javafx.application.Platform;
import javafx.event.Event;
import javafx.fxml.FXML;
import javafx.scene.control.Alert;
import javafx.scene.control.Button;
import javafx.scene.control.DialogPane;
import javafx.scene.control.Label;
import javafx.scene.control.TextArea;
import javafx.scene.image.Image;
import javafx.scene.image.ImageView;
import javafx.scene.Node;
import javafx.scene.paint.Color;
import javafx.stage.Stage;

public class NotificationController {
    public enum NotificationType {
        INFO,
        ERROR
    }

    private static final Logger logger = Logger.getLogger(Main.class.getName());

    private static final String informationString = "🛈";
    private static final String errorString = "⚠";

    private Runnable openCallback;

    @FXML
    TextArea descriptionTextArea;

    @FXML
    Label iconLabel;

    @FXML
    Button openButton;

    @FXML
    Label titleLabel;

    @FXML
    private ImageView iconImageView;

    public void handleOpenButtonPressed() {
        if (openCallback != null) {
            openCallback.run();
        }
    }

    public void handleDismissButtonPressed(Event event) {
        Node node = (Node) event.getSource();
        Stage stage = (Stage) node.getScene().getWindow();
        stage.close();
        Platform.exit();
    }

    public record NotificationParameters(String title,
                                         String description,
                                         NotificationType type,
                                         Optional<Runnable> openAction) {}

    public void setNotification(NotificationParameters params) throws IllegalArgumentException {
        titleLabel.setText(params.title());
        descriptionTextArea.setText(params.description());
        switch (params.type()) {
            case INFO:
                iconLabel.setText(informationString);
                iconLabel.setTextFill(Color.color(0.2, 0.47, 0.73));
                break;
            case ERROR:
                iconLabel.setText(errorString);
                break;
            default:
                throw new IllegalArgumentException("Invalid notification type: " + params.type());
        }

        if (params.openAction().isPresent()) {
            openButton.setVisible(true);
            this.openCallback = params.openAction().get();
        }
    }
}
